@@ -10,7 +10,7 @@ from contextlib import redirect_stdout
 import io
 import ast
 
-__version__ = '0.2.0'
+__version__ = '0.3.1'
 
 
 def ctrlc(signum, frame):
@@ -33,13 +33,17 @@ def helptext():
         Usage:  <JSON Data> | jello [OPTIONS] QUERY
 
                 -c    compact JSON output
+                -n    print selected null values
+                -r    raw string output (no quotes)
                 -v    version info
                 -h    help
 
         Use '_' as the input data and assign the result to 'r'. Use python dict syntax.
+        Use the lines() function for output suitable for a bash array.
 
         Example:
                 <JSON Data> | jello 'r = _["foo"]'
+                <JSON Data> | jello 'r = lines(_["foo"])'
     '''))
 
 
@@ -61,29 +65,65 @@ def print_json(data, compact=False):
         print(data)
 
 
-def process(data):
+def lines(data):
+    """
+    Convenience function to format each element on its own line for output suitable to be
+    assigned to a bash array.
+
+    Example:
+
+        r = lines(_["foo"])
+    """
+    return "\n".join(str(i) for i in data)
+
+
+def process(data, raw=None, nulls=None):
     result = None
     result_list = []
-    try:
-        if len(data.splitlines()) == 1:
-            try:
-                result = ast.literal_eval(data)
-            except Exception:
-                # if exception then it was not a list or dict
-                result = data.lstrip("'").rstrip("'")
-        else:
-            for entry in data.splitlines():
-                try:
-                    result_list.append(ast.literal_eval(entry))
-                except Exception:
-                    # if exception then it was not a list or dict
-                    result_list.append(entry.lstrip("'").rstrip("'"))
 
-    except Exception as e:
-        print(textwrap.dedent(f'''\
-            jello:  Exception - {e}
-            '''), file=sys.stderr)
-        sys.exit(1)
+    if data is None:
+        result = 'null' if nulls else ''
+
+    else:
+        try:
+            if len(data.splitlines()) == 1:
+                try:
+                    if data == 'True': result = 'true'
+                    elif data == 'False': result = 'false'
+                    elif data == 'None': result = 'null' if nulls else ''
+                    else: result = ast.literal_eval(data)
+
+                except (ValueError, SyntaxError):
+                    # if ValueError or SyntaxError exception then it was not a list or dict
+                    if raw:
+                        result = data
+                    else:
+                        result = '"' + data + '"'
+            else:
+                for entry in data.splitlines():
+                    try:
+                        if entry == 'True': result_list.append('true')
+                        elif entry == 'False': result_list.append('false')
+                        elif entry == 'None': 
+                            if nulls: result_list.append('null')
+                            else: result_list.append('')
+                        else: result_list.append(ast.literal_eval(entry))
+
+                    except (ValueError, SyntaxError):
+                        # if ValueError or SyntaxError exception then it was not a list or dict
+                        if raw:
+                            result_list.append(entry)
+                        else:
+                            result_list.append('"' + entry + '"')
+
+        except Exception as e:
+            print(textwrap.dedent(f'''\
+                jello:  Exception 100: {e}
+                        data: {data}
+                        result: {result}
+                        result_list: {result_list}
+                '''), file=sys.stderr)
+            sys.exit(1)
 
     if result_list:
         if isinstance(result_list[0], (dict, list)):
@@ -99,8 +139,8 @@ def process(data):
 
 def pyquery(data, query):
     _ = None
-    result = None
     query = 'r = None\n' + query + '\nprint(r)'
+    output = None
 
     # load the JSON or JSON Lines data
     try:
@@ -117,7 +157,7 @@ def pyquery(data, query):
             except Exception as e:
                 # can't parse the data. Throw a nice message and quit
                 print(textwrap.dedent(f'''\
-                    jello:  Exception - {e}
+                    jello:  Exception 110: {e}
                             Cannot parse line {i + 1} (Not JSON or JSON Lines data):
                             {str(jsonline)[:70]}
                     '''), file=sys.stderr)
@@ -153,23 +193,24 @@ def pyquery(data, query):
         sys.exit(1)
 
     except TypeError as e:
-        print(textwrap.dedent(f'''\
-            jello:  TypeError: {e}
-
-                    If joining a list to output as a bash array, consider:
-                    r = "\\n".join(str(i) for i in the_list)
-        '''), file=sys.stderr)
-        sys.exit(1)
+        if output is None:
+            pass
+        else:
+            print(textwrap.dedent(f'''\
+                jello:  TypeError: {e}
+            '''), file=sys.stderr)
+            sys.exit(1)
 
     except Exception as e:
         print(textwrap.dedent(f'''\
-            jello:  Exception - {e}
+            jello:  Exception 120: {e}
+                    _: {_}
+                    query: {query}
+                    f: {f.getvalue()}
         '''), file=sys.stderr)
         sys.exit(1)
 
-    result = process(output)
-
-    return result
+    return output
 
 
 def main():
@@ -196,6 +237,8 @@ def main():
             query = arg
 
     compact = 'c' in options
+    nulls = 'n' in options
+    raw = 'r' in options
     version_info = 'v' in options
     helpme = 'h' in options
 
@@ -208,7 +251,9 @@ def main():
     if stdin is None:
         print_error('jello:  missing piped JSON or JSON Lines data\n')
 
-    result = pyquery(stdin, query)
+    data = pyquery(stdin, query)
+
+    result = process(data, raw=raw, nulls=nulls)
 
     print_json(result, compact=compact)
 
